@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Context implementations
  *
- * Copyright (c) 2015 - 2023 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -32,10 +32,10 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "common.h"
 #include "compat.h"
 #include "hash_table.h"
 #include "in.h"
+#include "ly_common.h"
 #include "parser_data.h"
 #include "plugins_internal.h"
 #include "plugins_types.h"
@@ -81,48 +81,75 @@ static struct internal_modules_s {
 LIBYANG_API_DEF LY_ERR
 ly_ctx_set_searchdir(struct ly_ctx *ctx, const char *search_dir)
 {
+    int rc = LY_SUCCESS;
     struct stat st;
     char *new_dir = NULL;
+    uint32_t i;
+    LY_ARRAY_COUNT_TYPE u;
+    struct lys_module *mod;
 
     LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
 
-    if (search_dir) {
-        new_dir = realpath(search_dir, NULL);
-        LY_CHECK_ERR_RET(!new_dir,
-                LOGERR(ctx, LY_ESYS, "Unable to use search directory \"%s\" (%s).", search_dir, strerror(errno)),
-                LY_EINVAL);
-        if (strcmp(search_dir, new_dir)) {
-            LOGVRB("Search directory string \"%s\" canonized to \"%s\".", search_dir, new_dir);
+    if (!search_dir) {
+        /* fine, ignore */
+        goto cleanup;
+    }
+
+    new_dir = realpath(search_dir, NULL);
+    if (!new_dir) {
+        LOGERR(ctx, LY_ESYS, "Unable to use search directory \"%s\" (%s).", search_dir, strerror(errno)),
+        rc = LY_EINVAL;
+        goto cleanup;
+    }
+    if (strcmp(search_dir, new_dir)) {
+        LOGVRB("Search directory string \"%s\" canonized to \"%s\".", search_dir, new_dir);
+    }
+
+    if (access(new_dir, R_OK | X_OK)) {
+        LOGERR(ctx, LY_ESYS, "Unable to fully access search directory \"%s\" (%s).", new_dir, strerror(errno));
+        rc = LY_EINVAL;
+        goto cleanup;
+    }
+    if (stat(new_dir, &st)) {
+        LOGERR(ctx, LY_ESYS, "stat() failed for \"%s\" (%s).", new_dir, strerror(errno));
+        rc = LY_ESYS;
+        goto cleanup;
+    }
+    if (!S_ISDIR(st.st_mode)) {
+        LOGERR(ctx, LY_ESYS, "Given search directory \"%s\" is not a directory.", new_dir);
+        rc = LY_EINVAL;
+        goto cleanup;
+    }
+
+    /* avoid path duplication */
+    for (i = 0; i < ctx->search_paths.count; ++i) {
+        if (!strcmp(new_dir, ctx->search_paths.objs[i])) {
+            rc = LY_EEXIST;
+            goto cleanup;
         }
-        LY_CHECK_ERR_RET(access(new_dir, R_OK | X_OK),
-                LOGERR(ctx, LY_ESYS, "Unable to fully access search directory \"%s\" (%s).", new_dir, strerror(errno)); free(new_dir),
-                LY_EINVAL);
-        LY_CHECK_ERR_RET(stat(new_dir, &st),
-                LOGERR(ctx, LY_ESYS, "stat() failed for \"%s\" (%s).", new_dir, strerror(errno)); free(new_dir),
-                LY_ESYS);
-        LY_CHECK_ERR_RET(!S_ISDIR(st.st_mode),
-                LOGERR(ctx, LY_ESYS, "Given search directory \"%s\" is not a directory.", new_dir); free(new_dir),
-                LY_EINVAL);
-        /* avoid path duplication */
-        for (uint32_t u = 0; u < ctx->search_paths.count; ++u) {
-            if (!strcmp(new_dir, ctx->search_paths.objs[u])) {
-                free(new_dir);
-                return LY_EEXIST;
+    }
+    if (ly_set_add(&ctx->search_paths, new_dir, 1, NULL)) {
+        rc = LY_EMEM;
+        goto cleanup;
+    }
+
+    /* new searchdir - reset latests flags (possibly new revisions available) */
+    for (i = 0; i < ctx->list.count; ++i) {
+        mod = ctx->list.objs[i];
+
+        mod->latest_revision &= ~LYS_MOD_LATEST_SEARCHDIRS;
+        if (mod->parsed && mod->parsed->includes) {
+            for (u = 0; u < LY_ARRAY_COUNT(mod->parsed->includes); ++u) {
+                mod->parsed->includes[u].submodule->latest_revision &= ~LYS_MOD_LATEST_SEARCHDIRS;
             }
         }
-        if (ly_set_add(&ctx->search_paths, new_dir, 1, NULL)) {
-            free(new_dir);
-            return LY_EMEM;
-        }
-
-        /* new searchdir - possibly more latest revision available */
-        ly_ctx_reset_latests(ctx);
-
-        return LY_SUCCESS;
-    } else {
-        /* consider that no change is not actually an error */
-        return LY_SUCCESS;
     }
+
+cleanup:
+    if (rc) {
+        free(new_dir);
+    }
+    return rc;
 }
 
 LIBYANG_API_DEF const char * const *
@@ -239,6 +266,30 @@ ly_ctx_ht_err_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UN
     return !memcmp(&err1->tid, &err2->tid, sizeof err1->tid);
 }
 
+/**
+ * @brief Hash table value-equal callback for comparing leafref links hash table record.
+ */
+static ly_bool
+ly_ctx_ht_leafref_links_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(cb_data))
+{
+    struct lyd_leafref_links_rec *rec1 = val1_p, *rec2 = val2_p;
+
+    return rec1->node == rec2->node;
+}
+
+/**
+ * @brief Callback for freeing leafref links recorcd internal resources.
+ *
+ * @param[in] val_p Pointer to leafref links record
+ */
+static void
+ly_ctx_ht_leafref_links_rec_free(void *val_p)
+{
+    struct lyd_leafref_links_rec *rec = val_p;
+
+    lyd_free_leafref_links_rec(rec);
+}
+
 LIBYANG_API_DEF LY_ERR
 ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
 {
@@ -250,6 +301,7 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
     struct ly_in *in = NULL;
     LY_ERR rc = LY_SUCCESS;
     struct lys_glob_unres unres = {0};
+    ly_bool builtin_plugins_only;
 
     LY_CHECK_ARG_RET(NULL, new_ctx, LY_EINVAL);
 
@@ -260,7 +312,13 @@ ly_ctx_new(const char *search_dir, uint16_t options, struct ly_ctx **new_ctx)
     lydict_init(&ctx->dict);
 
     /* plugins */
-    LY_CHECK_ERR_GOTO(lyplg_init(), LOGINT(NULL); rc = LY_EINT, cleanup);
+    builtin_plugins_only = (options & LY_CTX_BUILTIN_PLUGINS_ONLY) ? 1 : 0;
+    LY_CHECK_ERR_GOTO(lyplg_init(builtin_plugins_only), LOGINT(NULL); rc = LY_EINT, cleanup);
+
+    if (options & LY_CTX_LEAFREF_LINKING) {
+        ctx->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec), ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
+        LY_CHECK_ERR_GOTO(!ctx->leafref_links_ht, rc = LY_EMEM, cleanup);
+    }
 
     /* initialize thread-specific error hash table */
     ctx->err_ht = lyht_new(1, sizeof(struct ly_ctx_err_rec), ly_ctx_ht_err_equal_cb, NULL, 1);
@@ -588,6 +646,18 @@ ly_ctx_set_options(struct ly_ctx *ctx, uint16_t option)
     LY_CHECK_ERR_RET((option & LY_CTX_NO_YANGLIBRARY) && !(ctx->flags & LY_CTX_NO_YANGLIBRARY),
             LOGARG(ctx, option), LY_EINVAL);
 
+    if (!(ctx->flags & LY_CTX_BUILTIN_PLUGINS_ONLY) && (option & LY_CTX_BUILTIN_PLUGINS_ONLY)) {
+        LOGERR(ctx, LY_EINVAL,
+                "Invalid argument %s (LY_CTX_BUILTIN_PLUGINS_ONLY can be set only when creating a new context) (%s()).",
+                "option", __func__);
+        return LY_EINVAL;
+    }
+
+    if (!(ctx->flags & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
+        ctx->leafref_links_ht = lyht_new(1, sizeof(struct lyd_leafref_links_rec), ly_ctx_ht_leafref_links_equal_cb, NULL, 1);
+        LY_CHECK_ERR_RET(!ctx->leafref_links_ht, LOGARG(ctx, option), LY_EMEM);
+    }
+
     if (!(ctx->flags & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
         ctx->flags |= LY_CTX_SET_PRIV_PARSED;
         /* recompile the whole context to set the priv pointers */
@@ -628,6 +698,11 @@ ly_ctx_unset_options(struct ly_ctx *ctx, uint16_t option)
     LY_CHECK_ARG_RET(ctx, ctx, LY_EINVAL);
     LY_CHECK_ERR_RET(option & LY_CTX_NO_YANGLIBRARY, LOGARG(ctx, option), LY_EINVAL);
 
+    if ((ctx->flags & LY_CTX_LEAFREF_LINKING) && (option & LY_CTX_LEAFREF_LINKING)) {
+        lyht_free(ctx->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
+        ctx->leafref_links_ht = NULL;
+    }
+
     if ((ctx->flags & LY_CTX_SET_PRIV_PARSED) && (option & LY_CTX_SET_PRIV_PARSED)) {
         struct lys_module *mod;
         uint32_t index;
@@ -646,7 +721,7 @@ ly_ctx_unset_options(struct ly_ctx *ctx, uint16_t option)
                 ext = &mod->compiled->exts[u];
                 LY_ARRAY_FOR(ext->substmts, v) {
                     if (ext->substmts[v].stmt & LY_STMT_DATA_NODE_MASK) {
-                        LY_LIST_FOR(*(struct lysc_node **)ext->substmts[v].storage, root) {
+                        LY_LIST_FOR(*VOIDPTR2_C(ext->substmts[v].storage), root) {
                             lysc_tree_dfs_full(root, lysc_node_clear_priv_dfs_cb, NULL);
                         }
                     }
@@ -720,6 +795,18 @@ ly_ctx_set_module_imp_clb(struct ly_ctx *ctx, ly_module_imp_clb clb, void *user_
 
     ctx->imp_clb = clb;
     ctx->imp_clb_data = user_data;
+
+    /* new import callback - reset latests flags (possibly new revisions available) */
+    for (uint32_t v = 0; v < ctx->list.count; ++v) {
+        struct lys_module *mod = ctx->list.objs[v];
+
+        mod->latest_revision &= ~LYS_MOD_LATEST_IMPCLB;
+        if (mod->parsed && mod->parsed->includes) {
+            for (LY_ARRAY_COUNT_TYPE u = 0; u < LY_ARRAY_COUNT(mod->parsed->includes); ++u) {
+                mod->parsed->includes[u].submodule->latest_revision &= ~LYS_MOD_LATEST_IMPCLB;
+            }
+        }
+    }
 }
 
 LIBYANG_API_DEF ly_ext_data_clb
@@ -1000,22 +1087,6 @@ ly_ctx_get_submodule2_latest(const struct lys_module *module, const char *submod
     return _ly_ctx_get_submodule2(module, submodule, NULL, 1);
 }
 
-LIBYANG_API_DEF void
-ly_ctx_reset_latests(struct ly_ctx *ctx)
-{
-    struct lys_module *mod;
-
-    for (uint32_t v = 0; v < ctx->list.count; ++v) {
-        mod = ctx->list.objs[v];
-        mod->latest_revision &= ~(LYS_MOD_LATEST_SEARCHDIRS | LYS_MOD_LATEST_IMPCLB);
-        if (mod->parsed && mod->parsed->includes) {
-            for (LY_ARRAY_COUNT_TYPE u = 0; u < LY_ARRAY_COUNT(mod->parsed->includes); ++u) {
-                mod->parsed->includes[u].submodule->latest_revision &= ~(LYS_MOD_LATEST_SEARCHDIRS | LYS_MOD_LATEST_IMPCLB);
-            }
-        }
-    }
-}
-
 LIBYANG_API_DEF uint32_t
 ly_ctx_internal_modules_count(const struct ly_ctx *ctx)
 {
@@ -1190,8 +1261,7 @@ ly_ctx_get_yanglib_data(const struct ly_ctx *ctx, struct lyd_node **root_p, cons
         LY_CHECK_GOTO(ret = ylib_deviation(cont, mod, 0), error);
 
         /* conformance-type */
-        LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "conformance-type", mod->implemented ? "implement" : "import", 0,
-                NULL), error);
+        LY_CHECK_GOTO(ret = lyd_new_term(cont, NULL, "conformance-type", mod->implemented ? "implement" : "import", 0, NULL), error);
 
         /* submodule list */
         LY_CHECK_GOTO(ret = ylib_submodules(cont, mod->parsed, 0), error);
@@ -1318,6 +1388,11 @@ ly_ctx_destroy(struct ly_ctx *ctx)
     /* leftover unres */
     lys_unres_glob_erase(&ctx->unres);
 
+    /* clean the leafref links hash table */
+    if (ctx->leafref_links_ht) {
+        lyht_free(ctx->leafref_links_ht, ly_ctx_ht_leafref_links_rec_free);
+    }
+
     /* clean the error hash table */
     lyht_free(ctx->err_ht, ly_ctx_ht_err_rec_free);
 
@@ -1327,7 +1402,11 @@ ly_ctx_destroy(struct ly_ctx *ctx)
     /* LYB hash lock */
     pthread_mutex_destroy(&ctx->lyb_hash_lock);
 
-    /* plugins - will be removed only if this is the last context */
+    /* context specific plugins */
+    ly_set_erase(&ctx->plugins_types, NULL);
+    ly_set_erase(&ctx->plugins_extensions, NULL);
+
+    /* shared plugins - will be removed only if this is the last context */
     lyplg_clean();
 
     free(ctx);
