@@ -20,13 +20,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
 #include "in_internal.h"
 #include "json.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_data.h"
 #include "parser_internal.h"
 #include "plugins_exts.h"
@@ -580,7 +580,7 @@ lydjson_metadata_finish(struct lyd_json_ctx *lydctx, struct lyd_node **first_p)
             continue;
         }
 
-        LOG_LOCSET(NULL, attr, NULL, NULL);
+        LOG_LOCSET(NULL, attr);
         log_location_items++;
 
         if (prev != meta_container->name.name) {
@@ -696,14 +696,14 @@ lydjson_metadata_finish(struct lyd_json_ctx *lydctx, struct lyd_node **first_p)
             lyd_free_tree(attr);
         }
 
-        LOG_LOCBACK(0, log_location_items, 0, 0);
+        LOG_LOCBACK(0, log_location_items);
         log_location_items = 0;
     }
 
 cleanup:
     lydict_remove(lydctx->jsonctx->ctx, prev);
 
-    LOG_LOCBACK(0, log_location_items, 0, 0);
+    LOG_LOCBACK(0, log_location_items);
     return ret;
 }
 
@@ -738,7 +738,9 @@ lydjson_metadata(struct lyd_json_ctx *lydctx, const struct lysc_node *snode, str
     assert(snode || node);
 
     nodetype = snode ? snode->nodetype : LYS_CONTAINER;
-    LOG_LOCSET(snode, NULL, NULL, NULL);
+    if (snode) {
+        LOG_LOCSET(snode, NULL);
+    }
 
     /* move to the second item in the name/X pair */
     LY_CHECK_GOTO(rc = lyjson_ctx_next(lydctx->jsonctx, &status), cleanup);
@@ -761,8 +763,8 @@ next_entry:
         LY_CHECK_GOTO((status != LYJSON_OBJECT) && (status != LYJSON_NULL), representation_error);
 
         if (!node || (node->schema != prev->schema)) {
-            LOGVAL(lydctx->jsonctx->ctx, LYVE_REFERENCE, "Missing JSON data instance #%u of %s:%s to be coupled with metadata.",
-                    instance, prev->schema->module->name, prev->schema->name);
+            LOGVAL(lydctx->jsonctx->ctx, LYVE_REFERENCE, "Missing JSON data instance #%" PRIu32
+                    " of %s:%s to be coupled with metadata.", instance, prev->schema->module->name, prev->schema->name);
             rc = LY_EVALID;
             goto cleanup;
         }
@@ -904,7 +906,7 @@ cleanup:
         }
     }
     free(dynamic_prefname);
-    LOG_LOCBACK(1, 0, 0, 0);
+    LOG_LOCBACK(snode ? 1 : 0, 0);
     return rc;
 }
 
@@ -1029,7 +1031,7 @@ lydjson_parse_opaq(struct lyd_json_ctx *lydctx, const char *name, size_t name_le
     LY_CHECK_GOTO(ret = lydjson_create_opaq(lydctx, name, name_len, prefix, prefix_len, parent, status_inner_p, node_p), cleanup);
 
     assert(*node_p);
-    LOG_LOCSET(NULL, *node_p, NULL, NULL);
+    LOG_LOCSET(NULL, *node_p);
 
     if ((*status_p == LYJSON_ARRAY) && (*status_inner_p == LYJSON_NULL)) {
         /* special array null value */
@@ -1071,17 +1073,19 @@ lydjson_parse_opaq(struct lyd_json_ctx *lydctx, const char *name, size_t name_le
         /* continue with the next instance */
         LY_CHECK_GOTO(ret = lyjson_ctx_next(lydctx->jsonctx, status_inner_p), cleanup);
         assert(*node_p);
-        lydjson_maintain_children(parent, first_p, node_p, lydctx->parse_opts & LYD_PARSE_ORDERED ? 1 : 0, NULL);
+        lydjson_maintain_children(parent, first_p, node_p,
+                lydctx->parse_opts & LYD_PARSE_ORDERED ? LYD_INSERT_NODE_LAST : LYD_INSERT_NODE_DEFAULT, NULL);
 
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
 
         LY_CHECK_GOTO(ret = lydjson_create_opaq(lydctx, name, name_len, prefix, prefix_len, parent, status_inner_p, node_p), cleanup);
 
         assert(*node_p);
-        LOG_LOCSET(NULL, *node_p, NULL, NULL);
+        LOG_LOCSET(NULL, *node_p);
     }
 
     if (*status_p == LYJSON_OBJECT) {
+        ((struct lyd_node_opaq *)*node_p)->hints |= LYD_NODEHINT_CONTAINER;
         /* process children */
         do {
             LY_CHECK_GOTO(ret = lydjson_subtree_r(lydctx, *node_p, lyd_node_child_p(*node_p), NULL), cleanup);
@@ -1095,7 +1099,7 @@ finish:
 
 cleanup:
     if (*node_p) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     return ret;
 }
@@ -1189,7 +1193,7 @@ lydjson_parse_attribute(struct lyd_json_ctx *lydctx, struct lyd_node *attr_node,
     /* parse as an attribute to a (opaque) node */
     if (!attr_node) {
         /* try to find the instance */
-        LY_LIST_FOR(*first_p, attr_node) {
+        LY_LIST_FOR(parent ? lyd_child(parent) : *first_p, attr_node) {
             if (snode) {
                 if (attr_node->schema) {
                     if (attr_node->schema == snode) {
@@ -1288,7 +1292,7 @@ lydjson_parse_any(struct lyd_json_ctx *lydctx, const struct lysc_node *snode, st
         LY_CHECK_ERR_GOTO(r, rc = r, cleanup);
 
         assert(*node);
-        LOG_LOCSET(NULL, *node, NULL, NULL);
+        LOG_LOCSET(NULL, *node);
         log_node = 1;
 
         /* parse any data tree with correct options, first backup the current options and then make the parser
@@ -1374,7 +1378,7 @@ lydjson_parse_any(struct lyd_json_ctx *lydctx, const struct lysc_node *snode, st
 
 cleanup:
     if (log_node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     lydctx->parse_opts = prev_parse_opts;
     lydctx->int_opts = prev_int_opts;
@@ -1409,7 +1413,7 @@ lydjson_parse_instance_inner(struct lyd_json_ctx *lydctx, const struct lysc_node
     LY_CHECK_RET(lyd_create_inner(snode, node));
 
     /* use it for logging */
-    LOG_LOCSET(NULL, *node, NULL, NULL);
+    LOG_LOCSET(NULL, *node);
 
     if (ext) {
         /* only parse these extension data and validate afterwards */
@@ -1447,7 +1451,7 @@ lydjson_parse_instance_inner(struct lyd_json_ctx *lydctx, const struct lysc_node
 
 cleanup:
     lydctx->parse_opts = prev_parse_opts;
-    LOG_LOCBACK(0, 1, 0, 0);
+    LOG_LOCBACK(0, 1);
     if (!(*node)->hash) {
         /* list without keys is unusable */
         lyd_free_tree(*node);
@@ -1483,12 +1487,15 @@ lydjson_parse_instance(struct lyd_json_ctx *lydctx, struct lyd_node *parent, str
     LY_ERR r, rc = LY_SUCCESS;
     uint32_t type_hints = 0;
 
-    LOG_LOCSET(snode, NULL, NULL, NULL);
+    LOG_LOCSET(snode, NULL);
 
     r = lydjson_data_check_opaq(lydctx, snode, &type_hints);
     if (r == LY_SUCCESS) {
         assert(snode->nodetype & (LYD_NODE_TERM | LYD_NODE_INNER | LYD_NODE_ANY));
-        if (snode->nodetype & LYD_NODE_TERM) {
+        if ((lydctx->parse_opts & LYD_PARSE_JSON_NULL) && (*status == LYJSON_NULL)) {
+            /* do not do anything if value is JSON 'null' */
+            goto cleanup;
+        } else if (snode->nodetype & LYD_NODE_TERM) {
             if ((*status != LYJSON_ARRAY) && (*status != LYJSON_NUMBER) && (*status != LYJSON_STRING) &&
                     (*status != LYJSON_FALSE) && (*status != LYJSON_TRUE) && (*status != LYJSON_NULL)) {
                 rc = LY_ENOT;
@@ -1548,7 +1555,7 @@ lydjson_parse_instance(struct lyd_json_ctx *lydctx, struct lyd_node *parent, str
     }
 
 cleanup:
-    LOG_LOCBACK(1, 0, 0, 0);
+    LOG_LOCBACK(1, 0);
     return rc;
 }
 
@@ -1740,7 +1747,8 @@ lydjson_subtree_r(struct lyd_json_ctx *lydctx, struct lyd_node *parent, struct l
                 }
                 LY_DPARSER_ERR_GOTO(r, rc = r, lydctx, cleanup);
 
-                lydjson_maintain_children(parent, first_p, &node, lydctx->parse_opts & LYD_PARSE_ORDERED ? 1 : 0, ext);
+                lydjson_maintain_children(parent, first_p, &node,
+                        lydctx->parse_opts & LYD_PARSE_ORDERED ? LYD_INSERT_NODE_LAST : LYD_INSERT_NODE_DEFAULT, ext);
 
                 /* move after the item(s) */
                 r = lyjson_ctx_next(lydctx->jsonctx, &status);
@@ -1778,7 +1786,8 @@ node_parsed:
     }
 
     /* finally connect the parsed node, is zeroed */
-    lydjson_maintain_children(parent, first_p, &node, lydctx->parse_opts & LYD_PARSE_ORDERED ? 1 : 0, ext);
+    lydjson_maintain_children(parent, first_p, &node,
+            lydctx->parse_opts & LYD_PARSE_ORDERED ? LYD_INSERT_NODE_LAST : LYD_INSERT_NODE_DEFAULT, ext);
 
     if (!parse_subtree) {
         /* move after the item(s) */
