@@ -20,7 +20,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
@@ -28,6 +27,7 @@
 #include "in.h"
 #include "in_internal.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_data.h"
 #include "parser_internal.h"
 #include "plugins_exts.h"
@@ -931,7 +931,8 @@ lyb_insert_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct lyd_
     if (parent && (LYD_CTX(parent) != LYD_CTX(node))) {
         lyplg_ext_insert(parent, node);
     } else {
-        lyd_insert_node(parent, first_p, node, lybctx->parse_opts & LYD_PARSE_ORDERED ? 1 : 0);
+        lyd_insert_node(parent, first_p, node,
+                lybctx->parse_opts & LYD_PARSE_ORDERED ? LYD_INSERT_NODE_LAST : LYD_INSERT_NODE_DEFAULT);
     }
     while (!parent && (*first_p)->prev->next) {
         *first_p = (*first_p)->prev;
@@ -1172,7 +1173,7 @@ lyb_parse_node_opaq(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct
     LY_CHECK_GOTO(ret, cleanup);
 
     assert(node);
-    LOG_LOCSET(NULL, node, NULL, NULL);
+    LOG_LOCSET(NULL, node);
 
     /* process children */
     ret = lyb_parse_siblings(lybctx, node, NULL, NULL);
@@ -1181,11 +1182,11 @@ lyb_parse_node_opaq(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct
     /* register parsed opaq node */
     lyb_finish_opaq(lybctx, parent, flags, &attr, &node, first_p, parsed);
     assert(!attr && !node);
-    LOG_LOCBACK(0, 1, 0, 0);
+    LOG_LOCBACK(0, 1);
 
 cleanup:
     if (node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     free(prefix);
     free(module_key);
@@ -1273,12 +1274,12 @@ lyb_parse_node_any(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const st
     }
 
     assert(node);
-    LOG_LOCSET(NULL, node, NULL, NULL);
+    LOG_LOCSET(NULL, node);
 
     /* register parsed anydata node */
     lyb_finish_node(lybctx, parent, flags, &meta, &node, first_p, parsed);
 
-    LOG_LOCBACK(0, 1, 0, 0);
+    LOG_LOCBACK(0, 1);
     return LY_SUCCESS;
 
 error:
@@ -1316,7 +1317,7 @@ lyb_parse_node_inner(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const
     LY_CHECK_GOTO(ret, error);
 
     assert(node);
-    LOG_LOCSET(NULL, node, NULL, NULL);
+    LOG_LOCSET(NULL, node);
 
     /* process children */
     ret = lyb_parse_siblings(lybctx, node, NULL, NULL);
@@ -1334,12 +1335,12 @@ lyb_parse_node_inner(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const
     /* register parsed node */
     lyb_finish_node(lybctx, parent, flags, &meta, &node, first_p, parsed);
 
-    LOG_LOCBACK(0, 1, 0, 0);
+    LOG_LOCBACK(0, 1);
     return LY_SUCCESS;
 
 error:
     if (node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     lyd_free_meta_siblings(meta);
     lyd_free_tree(node);
@@ -1374,11 +1375,11 @@ lyb_parse_node_leaf(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
     LY_CHECK_GOTO(ret, error);
 
     assert(node);
-    LOG_LOCSET(NULL, node, NULL, NULL);
+    LOG_LOCSET(NULL, node);
 
     lyb_finish_node(lybctx, parent, flags, &meta, &node, first_p, parsed);
 
-    LOG_LOCBACK(0, 1, 0, 0);
+    LOG_LOCBACK(0, 1);
     return LY_SUCCESS;
 
 error:
@@ -1454,7 +1455,7 @@ lyb_parse_node_list(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
         LY_CHECK_GOTO(ret, error);
 
         assert(node);
-        LOG_LOCSET(NULL, node, NULL, NULL);
+        LOG_LOCSET(NULL, node);
         log_node = 1;
 
         /* process children */
@@ -1473,7 +1474,7 @@ lyb_parse_node_list(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
         /* register parsed list node */
         lyb_finish_node(lybctx, parent, flags, &meta, &node, first_p, parsed);
 
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
         log_node = 0;
     }
 
@@ -1485,7 +1486,7 @@ lyb_parse_node_list(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
 
 error:
     if (log_node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     lyd_free_meta_siblings(meta);
     lyd_free_tree(node);
