@@ -20,12 +20,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
 #include "in_internal.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_data.h"
 #include "parser_internal.h"
 #include "plugins_exts.h"
@@ -75,7 +75,7 @@ lydxml_metadata(struct lyd_xml_ctx *lydctx, const struct lysc_node *sparent, str
 
     *meta = NULL;
 
-    LOG_LOCSET(sparent, NULL, NULL, NULL);
+    LOG_LOCSET(sparent, NULL);
 
     /* check for NETCONF filter unqualified attributes */
     if (!strcmp(sparent->module->name, "notifications")) {
@@ -165,7 +165,7 @@ create_meta:
     }
 
 cleanup:
-    LOG_LOCBACK(1, 0, 0, 0);
+    LOG_LOCBACK(1, 0);
     if (ret) {
         lyd_free_meta_siblings(*meta);
         *meta = NULL;
@@ -457,7 +457,7 @@ lydxml_get_hints_opaq(const char *name, size_t name_len, const char *value, size
         if ((unsigned)(ptr - value) == value_len) {
             /* number value */
             *hints |= LYD_VALHINT_DECNUM;
-            if ((num < INT32_MIN) || (num > UINT32_MAX)) {
+            if ((num < INT32_MIN) || (num > (long)UINT32_MAX)) {
                 /* large number */
                 *hints |= LYD_VALHINT_NUM64;
             }
@@ -481,7 +481,7 @@ lydxml_get_hints_opaq(const char *name, size_t name_len, const char *value, size
         opaq = (struct lyd_node_opaq *)first;
         assert(opaq->format == LY_VALUE_XML);
         if (!ly_strncmp(opaq->name.name, name, name_len) &&
-                ((ns && !strcmp(opaq->name.module_ns, ns)) || (!ns && !opaq->name.module_ns))) {
+                ((ns && opaq->name.module_ns && !strcmp(opaq->name.module_ns, ns)) || (!ns && !opaq->name.module_ns))) {
             if (opaq->value && opaq->value[0]) {
                 /* leaf-list nodes */
                 opaq->hints |= LYD_NODEHINT_LEAFLIST;
@@ -672,7 +672,7 @@ lydxml_subtree_opaq(struct lyd_xml_ctx *lydctx, const struct lyd_node *sibling,
     LY_CHECK_GOTO(rc, cleanup);
 
     assert(*node);
-    LOG_LOCSET(NULL, *node, NULL, NULL);
+    LOG_LOCSET(NULL, *node);
 
     /* parser next */
     rc = lyxml_ctx_next(xmlctx);
@@ -709,7 +709,7 @@ lydxml_subtree_opaq(struct lyd_xml_ctx *lydctx, const struct lyd_node *sibling,
 
 cleanup:
     if (*node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     ly_free_prefix_data(format, val_prefix_data);
     if (dynamic) {
@@ -747,7 +747,7 @@ lydxml_subtree_term(struct lyd_xml_ctx *lydctx, struct lyd_node *parent, const s
     LY_DPARSER_ERR_GOTO(r, rc = r, lydctx, cleanup);
 
     if (*node) {
-        LOG_LOCSET(NULL, *node, NULL, NULL);
+        LOG_LOCSET(NULL, *node);
     }
 
     if (*node && parent && (snode->flags & LYS_KEY)) {
@@ -778,7 +778,7 @@ lydxml_subtree_term(struct lyd_xml_ctx *lydctx, struct lyd_node *parent, const s
 
 cleanup:
     if (*node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     if (rc && (!(lydctx->val_opts & LYD_VALIDATE_MULTI_ERROR) || (rc != LY_EVALID))) {
         lyd_free_tree(*node);
@@ -819,7 +819,7 @@ lydxml_subtree_inner(struct lyd_xml_ctx *lydctx, const struct lysc_node *snode,
     LY_CHECK_GOTO(rc, cleanup);
 
     assert(*node);
-    LOG_LOCSET(NULL, *node, NULL, NULL);
+    LOG_LOCSET(NULL, *node);
 
     /* parser next */
     rc = lyxml_ctx_next(xmlctx);
@@ -863,7 +863,7 @@ lydxml_subtree_inner(struct lyd_xml_ctx *lydctx, const struct lysc_node *snode,
 
 cleanup:
     if (*node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     lydctx->parse_opts = prev_parse_opts;
     if (rc && ((*node && !(*node)->hash) || !(lydctx->val_opts & LYD_VALIDATE_MULTI_ERROR) || (rc != LY_EVALID))) {
@@ -923,7 +923,7 @@ lydxml_subtree_any(struct lyd_xml_ctx *lydctx, const struct lysc_node *snode, co
         LY_CHECK_ERR_GOTO(r, rc = r, cleanup);
 
         assert(*node);
-        LOG_LOCSET(NULL, *node, NULL, NULL);
+        LOG_LOCSET(NULL, *node);
         log_node = 1;
 
         /* parser next */
@@ -948,7 +948,7 @@ lydxml_subtree_any(struct lyd_xml_ctx *lydctx, const struct lysc_node *snode, co
 
 cleanup:
     if (log_node) {
-        LOG_LOCBACK(0, 1, 0, 0);
+        LOG_LOCBACK(0, 1);
     }
     lydctx->parse_opts = prev_parse_opts;
     lydctx->int_opts = prev_int_opts;
@@ -1121,7 +1121,8 @@ node_parsed:
         r = lyplg_ext_insert(parent, node);
         LY_CHECK_ERR_GOTO(r, rc = r; lyd_free_tree(node), cleanup);
     } else {
-        lyd_insert_node(parent, first_p, node, lydctx->parse_opts & LYD_PARSE_ORDERED ? 1 : 0);
+        lyd_insert_node(parent, first_p, node,
+                lydctx->parse_opts & LYD_PARSE_ORDERED ? LYD_INSERT_NODE_LAST : LYD_INSERT_NODE_DEFAULT);
     }
     while (!parent && (*first_p)->prev->next) {
         *first_p = (*first_p)->prev;
@@ -1349,7 +1350,7 @@ lydxml_env_netconf_rpc(struct lyxml_ctx *xmlctx, struct lyd_node **envp, uint32_
     r = lydxml_envelope(xmlctx, "action", "urn:ietf:params:xml:ns:yang:1", 0, &child);
     if (r == LY_SUCCESS) {
         /* insert */
-        lyd_insert_node(*envp, NULL, child, 0);
+        lyd_insert_node(*envp, NULL, child, LYD_INSERT_NODE_DEFAULT);
 
         /* NETCONF action */
         *int_opts = LYD_INTOPT_NO_SIBLINGS | LYD_INTOPT_ACTION;
@@ -1438,7 +1439,7 @@ lydxml_opaq_r(struct lyxml_ctx *xmlctx, struct lyd_node *parent)
     }
 
     /* insert */
-    lyd_insert_node(parent, NULL, node, 1);
+    lyd_insert_node(parent, NULL, node, LYD_INSERT_NODE_LAST);
 
     /* update the value */
     opaq = (struct lyd_node_opaq *)node;
@@ -1572,7 +1573,7 @@ check_child:
         LY_CHECK_GOTO(r = lyxml_ctx_next(xmlctx), error);
 
         /* insert */
-        lyd_insert_node(parent, NULL, child, 1);
+        lyd_insert_node(parent, NULL, child, LYD_INSERT_NODE_LAST);
     }
 
     return LY_SUCCESS;
@@ -1747,7 +1748,7 @@ check_child:
         LY_CHECK_GOTO(r = lyxml_ctx_next(xmlctx), error);
 
         /* insert */
-        lyd_insert_node(parent, NULL, child, 1);
+        lyd_insert_node(parent, NULL, child, LYD_INSERT_NODE_LAST);
     }
 
     return LY_SUCCESS;
@@ -1791,7 +1792,7 @@ lydxml_env_netconf_reply(struct lyxml_ctx *xmlctx, struct lyd_node **envp, uint3
     r = lydxml_envelope(xmlctx, "ok", "urn:ietf:params:xml:ns:netconf:base:1.0", 0, &child);
     if (r == LY_SUCCESS) {
         /* insert */
-        lyd_insert_node(*envp, NULL, child, 1);
+        lyd_insert_node(*envp, NULL, child, LYD_INSERT_NODE_LAST);
 
         /* finish child parsing */
         if (xmlctx->status != LYXML_ELEM_CLOSE) {
@@ -1822,7 +1823,7 @@ lydxml_env_netconf_reply(struct lyxml_ctx *xmlctx, struct lyd_node **envp, uint3
         }
 
         /* insert */
-        lyd_insert_node(*envp, NULL, child, 1);
+        lyd_insert_node(*envp, NULL, child, LYD_INSERT_NODE_LAST);
 
         /* parse all children of "rpc-error" */
         LY_CHECK_GOTO(rc = lydxml_env_netconf_rpc_reply_error(xmlctx, child), cleanup);
