@@ -24,11 +24,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
+#include "ly_common.h"
 #include "path.h"
+#include "plugins_internal.h"
 #include "schema_compile.h"
 #include "set.h"
 #include "tree.h"
@@ -46,27 +47,23 @@ static const struct lys_module *
 ly_schema_resolve_prefix(const struct ly_ctx *UNUSED(ctx), const char *prefix, size_t prefix_len, const void *prefix_data)
 {
     const struct lysp_module *prefix_mod = prefix_data;
-    struct lys_module *m = NULL;
     LY_ARRAY_COUNT_TYPE u;
     const char *local_prefix;
 
     local_prefix = prefix_mod->is_submod ? ((struct lysp_submodule *)prefix_mod)->prefix : prefix_mod->mod->prefix;
     if (!prefix_len || !ly_strncmp(local_prefix, prefix, prefix_len)) {
         /* it is the prefix of the module itself */
-        m = prefix_mod->mod;
+        return prefix_mod->mod;
     }
 
     /* search in imports */
-    if (!m) {
-        LY_ARRAY_FOR(prefix_mod->imports, u) {
-            if (!ly_strncmp(prefix_mod->imports[u].prefix, prefix, prefix_len)) {
-                m = prefix_mod->imports[u].module;
-                break;
-            }
+    LY_ARRAY_FOR(prefix_mod->imports, u) {
+        if (!ly_strncmp(prefix_mod->imports[u].prefix, prefix, prefix_len)) {
+            return prefix_mod->imports[u].module;
         }
     }
 
-    return m;
+    return NULL;
 }
 
 /**
@@ -305,15 +302,40 @@ lyplg_type_get_prefix(const struct lys_module *mod, LY_VALUE_FORMAT format, void
 }
 
 LIBYANG_API_DEF LY_ERR
-lyplg_type_compare_simple(const struct lyd_value *val1, const struct lyd_value *val2)
+lyplg_type_compare_simple(const struct ly_ctx *ctx, const struct lyd_value *val1, const struct lyd_value *val2)
 {
-    if (val1->_canonical == val2->_canonical) {
+    const char *can1, *can2;
+
+    can1 = lyd_value_get_canonical(ctx, val1);
+    can2 = lyd_value_get_canonical(ctx, val2);
+    if (can1 == can2) {
         return LY_SUCCESS;
     }
 
     return LY_ENOT;
 }
 
+LIBYANG_API_DEF int
+lyplg_type_sort_simple(const struct ly_ctx *ctx, const struct lyd_value *val1, const struct lyd_value *val2)
+{
+    int cmp;
+    const char *can1, *can2;
+
+    can1 = lyd_value_get_canonical(ctx, val1);
+    can2 = lyd_value_get_canonical(ctx, val2);
+    if (can1 && can2) {
+        cmp = strcmp(can1, can2);
+        return cmp;
+    } else if (!can1 && can2) {
+        return -1;
+    } else if (can1 && !can2) {
+        return 1;
+    } else {
+        /* !can1 && !can2 */
+        return 0;
+    }
+}
+
 LIBYANG_API_DEF const void *
 lyplg_type_print_simple(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT UNUSED(format),
         void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
@@ -830,7 +852,7 @@ lyplg_type_lypath_new(const struct ly_ctx *ctx, const char *value, size_t value_
     }
 
     /* remember the current last error */
-    e = ly_err_last(ctx);
+    e = (struct ly_err_item *)ly_err_last(ctx);
 
     /* parse the value */
     ret = ly_path_parse(ctx, ctx_node, value, value_len, 0, LY_PATH_BEGIN_ABSOLUTE, prefix_opt, LY_PATH_PRED_SIMPLE, &exp);
@@ -860,7 +882,7 @@ cleanup:
     lyxp_expr_free(ctx, exp);
     if (ret) {
         /* generate error, spend the context error, if any */
-        e = e ? e->next : ly_err_last(ctx);
+        e = e ? e->next : (struct ly_err_item *)ly_err_last(ctx);
         ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, err_fmt, (int)value_len, value, e ? ": " : ".", e ? e->msg : "");
         if (e) {
             ly_err_clean((struct ly_ctx *)ctx, e);
@@ -992,10 +1014,11 @@ cleanup:
 
 LIBYANG_API_DEF LY_ERR
 lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct lyd_node *node, struct lyd_value *value,
-        const struct lyd_node *tree, struct lyd_node **target, char **errmsg)
+        const struct lyd_node *tree, struct ly_set **targets, char **errmsg)
 {
     LY_ERR rc = LY_SUCCESS;
     struct lyxp_expr *target_path = NULL;
+    const struct ly_err_item *e;
     struct lyxp_set set = {0};
     const char *val_str, *xp_err_msg;
     uint32_t i;
@@ -1003,8 +1026,9 @@ lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct ly
 
     LY_CHECK_ARG_RET(NULL, lref, node, value, errmsg, LY_EINVAL);
 
-    if (target) {
-        *target = NULL;
+    *errmsg = NULL;
+    if (targets) {
+        *targets = NULL;
     }
 
     /* get the canonical value */
@@ -1026,8 +1050,9 @@ lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct ly
     rc = lyxp_eval(LYD_CTX(node), target_path ? target_path : lref->path, node->schema->module,
             LY_VALUE_SCHEMA_RESOLVED, lref->prefixes, node, node, tree, NULL, &set, LYXP_IGNORE_WHEN);
     if (rc) {
-        if (ly_errcode(LYD_CTX(node)) == rc) {
-            xp_err_msg = ly_errmsg(LYD_CTX(node));
+        e = ly_err_last(LYD_CTX(node));
+        if (e && (e->err == rc)) {
+            xp_err_msg = e->msg;
         } else {
             xp_err_msg = NULL;
         }
@@ -1058,7 +1083,7 @@ lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct ly
                 continue;
             }
 
-            if (!lref->plugin->compare(&((struct lyd_node_term *)set.val.nodes[i].node)->value, value)) {
+            if (!lref->plugin->compare(LYD_CTX(node), &((struct lyd_node_term *)set.val.nodes[i].node)->value, value)) {
                 break;
             }
         }
@@ -1073,8 +1098,21 @@ lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct ly
         }
         goto cleanup;
     }
-    if (target) {
-        *target = set.val.nodes[i].node;
+    if (targets) {
+        LY_CHECK_GOTO(rc = ly_set_new(targets), cleanup);
+        for (i = 0; i < set.used; ++i) {
+            if (set.val.nodes[i].type != LYXP_NODE_ELEM) {
+                continue;
+            }
+            if (((struct lyd_node_term *)set.val.nodes[i].node)->value.realtype != value->realtype) {
+                continue;
+            }
+
+            if (!lref->plugin->compare(LYD_CTX(node), &((struct lyd_node_term *)set.val.nodes[i].node)->value, value)) {
+                rc = ly_set_add(*targets, set.val.nodes[i].node, 0, NULL);
+                LY_CHECK_GOTO(rc, cleanup);
+            }
+        }
     }
 
 cleanup:
