@@ -24,12 +24,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
 #include "in.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_schema.h"
 #include "path.h"
 #include "plugins.h"
@@ -88,9 +88,11 @@ remove_nodelevel:
         if (nextlevel != 2) {
             if ((parent_module && (parent_module == ctx->cur_mod)) || (!parent_module && (ctx->path_len > 1) && (name[0] == '{'))) {
                 /* module not changed, print the name unprefixed */
-                len = snprintf(&ctx->path[ctx->path_len], LYSC_CTX_BUFSIZE - ctx->path_len, "%s%s", nextlevel ? "/" : "", name);
+                len = snprintf(&ctx->path[ctx->path_len], LYSC_CTX_BUFSIZE - ctx->path_len, "%s%s",
+                        nextlevel ? "/" : "", name);
             } else {
-                len = snprintf(&ctx->path[ctx->path_len], LYSC_CTX_BUFSIZE - ctx->path_len, "%s%s:%s", nextlevel ? "/" : "", ctx->cur_mod->name, name);
+                len = snprintf(&ctx->path[ctx->path_len], LYSC_CTX_BUFSIZE - ctx->path_len, "%s%s:%s",
+                        nextlevel ? "/" : "", ctx->cur_mod->name, name);
             }
         } else {
             len = snprintf(&ctx->path[ctx->path_len], LYSC_CTX_BUFSIZE - ctx->path_len, "='%s'}", name);
@@ -103,8 +105,8 @@ remove_nodelevel:
         }
     }
 
-    LOG_LOCBACK(0, 0, 1, 0);
-    LOG_LOCSET(NULL, NULL, ctx->path, NULL);
+    ly_log_location_revert(0, 0, 1, 0);
+    ly_log_location(NULL, NULL, ctx->path, NULL);
 }
 
 /**
@@ -127,7 +129,6 @@ lys_compile_extension(struct lysc_ctx *ctx, struct lysp_ext_instance *extp, stru
 
         /* compile the extension definition */
         *ext = ep->compiled = calloc(1, sizeof **ext);
-        (*ext)->refcount = 1;
         DUP_STRING_GOTO(ctx->ctx, ep->name, (*ext)->name, ret, cleanup);
         DUP_STRING_GOTO(ctx->ctx, ep->argname, (*ext)->argname, ret, cleanup);
         LY_CHECK_GOTO(ret = lysp_ext_find_definition(ctx->ctx, extp, (const struct lys_module **)&(*ext)->module, NULL),
@@ -524,7 +525,7 @@ lys_compile_unres_when_cyclic(struct lyxp_set *set, const struct lysc_node *node
         do {
             struct lysc_when **when_list, *when;
 
-            LOG_LOCSET(node, NULL, NULL, NULL);
+            LOG_LOCSET(node, NULL);
             when_list = lysc_node_when(node);
             LY_ARRAY_FOR(when_list, u) {
                 when = when_list[u];
@@ -532,7 +533,7 @@ lys_compile_unres_when_cyclic(struct lyxp_set *set, const struct lysc_node *node
                         when->context, when->context, &tmp_set, LYXP_SCNODE_SCHEMA);
                 if (ret != LY_SUCCESS) {
                     LOGVAL(set->ctx, LYVE_SEMANTICS, "Invalid when condition \"%s\".", when->cond->expr);
-                    LOG_LOCBACK(1, 0, 0, 0);
+                    LOG_LOCBACK(1, 0);
                     goto cleanup;
                 }
 
@@ -549,7 +550,7 @@ lys_compile_unres_when_cyclic(struct lyxp_set *set, const struct lysc_node *node
                         LOGVAL(set->ctx, LYVE_SEMANTICS, "When condition cyclic dependency on the node \"%s\".",
                                 tmp_set.val.scnodes[j].scnode->name);
                         ret = LY_EVALID;
-                        LOG_LOCBACK(1, 0, 0, 0);
+                        LOG_LOCBACK(1, 0);
                         goto cleanup;
                     }
 
@@ -566,14 +567,14 @@ lys_compile_unres_when_cyclic(struct lyxp_set *set, const struct lysc_node *node
                     } else {
                         /* context node was traversed, so just add the dependent node */
                         ret = lyxp_set_scnode_insert_node(&tmp_set, node, LYXP_SET_SCNODE_START_USED, LYXP_AXIS_CHILD, NULL);
-                        LY_CHECK_ERR_GOTO(ret, LOG_LOCBACK(1, 0, 0, 0), cleanup);
+                        LY_CHECK_ERR_GOTO(ret, LOG_LOCBACK(1, 0), cleanup);
                     }
                 }
 
                 /* merge this set into the global when set */
                 lyxp_set_scnode_merge(set, &tmp_set);
             }
-            LOG_LOCBACK(1, 0, 0, 0);
+            LOG_LOCBACK(1, 0);
 
             /* check when of non-data parents as well */
             node = node->parent;
@@ -707,7 +708,7 @@ lys_compile_unres_must(struct lysc_ctx *ctx, const struct lysc_node *node, const
     LY_ERR ret = LY_SUCCESS;
     uint16_t flg;
 
-    LOG_LOCSET(node, NULL, NULL, NULL);
+    LOG_LOCSET(node, NULL);
 
     memset(&tmp_set, 0, sizeof tmp_set);
     opts = LYXP_SCNODE_SCHEMA | ((node->flags & LYS_IS_OUTPUT) ? LYXP_SCNODE_OUTPUT : 0);
@@ -751,7 +752,7 @@ lys_compile_unres_must(struct lysc_ctx *ctx, const struct lysc_node *node, const
 
 cleanup:
     lyxp_set_free_content(&tmp_set);
-    LOG_LOCBACK(1, 0, 0, 0);
+    LOG_LOCBACK(1, 0);
     return ret;
 }
 
@@ -947,14 +948,14 @@ lys_compile_unres_dflt(struct lysc_ctx *ctx, struct lysc_node *node, struct lysc
     }
 
     if (ret) {
-        LOG_LOCSET(node, NULL, NULL, NULL);
+        LOG_LOCSET(node, NULL);
         if (err) {
             LOGVAL(ctx->ctx, LYVE_SEMANTICS, "Invalid default - value does not fit the type (%s).", err->msg);
             ly_err_free(err);
         } else {
             LOGVAL(ctx->ctx, LYVE_SEMANTICS, "Invalid default - value does not fit the type.");
         }
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         return ret;
     }
 
@@ -1045,7 +1046,7 @@ lys_compile_unres_llist_dflts(struct lysc_ctx *ctx, struct lysc_node_leaflist *l
         /* configuration data values must be unique - so check the default values */
         for (u = orig_count; u < LY_ARRAY_COUNT(llist->dflts); ++u) {
             for (v = 0; v < u; ++v) {
-                if (!llist->dflts[u]->realtype->plugin->compare(llist->dflts[u], llist->dflts[v])) {
+                if (!llist->dflts[u]->realtype->plugin->compare(ctx->ctx, llist->dflts[u], llist->dflts[v])) {
                     lysc_update_path(ctx, llist->parent ? llist->parent->module : NULL, llist->name);
                     LOGVAL(ctx->ctx, LYVE_SEMANTICS, "Configuration leaf-list has multiple defaults of the same value \"%s\".",
                             (char *)llist->dflts[u]->realtype->plugin->print(ctx->ctx, llist->dflts[u], LY_VALUE_CANON,
@@ -1071,7 +1072,7 @@ lys_compile_unres_llist_dflts(struct lysc_ctx *ctx, struct lysc_node_leaflist *l
  * @return Pointer to the leafref or next leafref, otherwise NULL.
  */
 static struct lysc_type_leafref *
-lys_type_leafref_next(const struct lysc_node *node, uint64_t *index)
+lys_type_leafref_next(const struct lysc_node *node, LY_ARRAY_COUNT_TYPE *index)
 {
     struct lysc_type_leafref *ret = NULL;
     struct lysc_type_union *uni;
@@ -1207,6 +1208,76 @@ implement_all:
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Check that a disabled node (to be freed) can be freed and is not referenced.
+ *
+ * @param[in] node Disabled node to check.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+lys_compile_unres_check_disabled(const struct lysc_node *node)
+{
+    const struct lysc_node *parent;
+    struct lysc_node_list *slist;
+    LY_ARRAY_COUNT_TYPE u, v;
+    int found;
+
+    if (node->flags & LYS_KEY) {
+        LOG_LOCSET(node, NULL);
+        LOGVAL(node->module->ctx, LYVE_REFERENCE, "Key \"%s\" is disabled.", node->name);
+        LOG_LOCBACK(1, 0);
+        return LY_EVALID;
+    }
+
+    for (parent = node->parent; parent; parent = parent->parent) {
+        if (parent->nodetype != LYS_LIST) {
+            continue;
+        }
+
+        /* check and fix list uniques */
+        slist = (struct lysc_node_list *)parent;
+        found = 0;
+        LY_ARRAY_FOR(slist->uniques, u) {
+            LY_ARRAY_FOR(slist->uniques[u], v) {
+                if (slist->uniques[u][v] == (struct lysc_node_leaf *)node) {
+                    found = 1;
+                    break;
+                }
+            }
+
+            if (found) {
+                break;
+            }
+        }
+
+        if (found) {
+            if (LY_ARRAY_COUNT(slist->uniques[u]) > 1) {
+                /* remove the item */
+                if (v < LY_ARRAY_COUNT(slist->uniques[u]) - 1) {
+                    memmove(&slist->uniques[u][v], &slist->uniques[u][v + 1],
+                            (LY_ARRAY_COUNT(slist->uniques[u]) - v - 1) * sizeof slist->uniques[u][v]);
+                }
+                LY_ARRAY_DECREMENT(slist->uniques[u]);
+            } else {
+                /* remove the whole unique array */
+                LY_ARRAY_FREE(slist->uniques[u]);
+                if (LY_ARRAY_COUNT(slist->uniques) > 1) {
+                    if (u < LY_ARRAY_COUNT(slist->uniques) - 1) {
+                        memmove(&slist->uniques[u], &slist->uniques[u + 1],
+                                (LY_ARRAY_COUNT(slist->uniques) - u - 1) * sizeof slist->uniques[u]);
+                    }
+                    LY_ARRAY_DECREMENT(slist->uniques);
+                } else {
+                    LY_ARRAY_FREE(slist->uniques);
+                    slist->uniques = NULL;
+                }
+            }
+        }
+    }
+
+    return LY_SUCCESS;
+}
+
 /**
  * @brief Finish dependency set compilation by resolving all the unres sets.
  *
@@ -1246,12 +1317,12 @@ resolve_all:
         l = ds_unres->disabled_leafrefs.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, l->node->module->parsed, l->ext);
 
-        LOG_LOCSET(l->node, NULL, NULL, NULL);
+        LOG_LOCSET(l->node, NULL);
         v = 0;
         while ((ret == LY_SUCCESS) && (lref = lys_type_leafref_next(l->node, &v))) {
             ret = lys_compile_unres_leafref(&cctx, l->node, lref, l->local_mod);
         }
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
 
         ly_set_rm_index(&ds_unres->disabled_leafrefs, i, free);
@@ -1265,12 +1336,12 @@ resolve_all:
         l = ds_unres->leafrefs.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, l->node->module->parsed, l->ext);
 
-        LOG_LOCSET(l->node, NULL, NULL, NULL);
+        LOG_LOCSET(l->node, NULL);
         v = 0;
         while ((ret == LY_SUCCESS) && (lref = lys_type_leafref_next(l->node, &v))) {
             ret = lys_compile_unres_leafref(&cctx, l->node, lref, l->local_mod);
         }
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
     }
     for (i = processed_leafrefs; i < ds_unres->leafrefs.count; ++i) {
@@ -1298,9 +1369,9 @@ resolve_all:
         w = ds_unres->whens.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, w->node->module->parsed, NULL);
 
-        LOG_LOCSET(w->node, NULL, NULL, NULL);
+        LOG_LOCSET(w->node, NULL);
         ret = lys_compile_unres_when(&cctx, w->when, w->node);
-        LOG_LOCBACK(w->node ? 1 : 0, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
 
         free(w);
@@ -1313,9 +1384,9 @@ resolve_all:
         m = ds_unres->musts.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, m->node->module->parsed, m->ext);
 
-        LOG_LOCSET(m->node, NULL, NULL, NULL);
+        LOG_LOCSET(m->node, NULL);
         ret = lys_compile_unres_must(&cctx, m->node, m->local_mods);
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
 
         lysc_unres_must_free(m);
@@ -1328,9 +1399,9 @@ resolve_all:
         node = ds_unres->disabled_bitenums.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, node->module->parsed, NULL);
 
-        LOG_LOCSET(node, NULL, NULL, NULL);
+        LOG_LOCSET(node, NULL);
         ret = lys_compile_unres_disabled_bitenum(&cctx, (struct lysc_node_leaf *)node);
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
 
         ly_set_rm_index(&ds_unres->disabled_bitenums, i, NULL);
@@ -1342,13 +1413,13 @@ resolve_all:
         d = ds_unres->dflts.objs[i];
         LYSC_CTX_INIT_PMOD(cctx, d->leaf->module->parsed, NULL);
 
-        LOG_LOCSET(&d->leaf->node, NULL, NULL, NULL);
+        LOG_LOCSET(&d->leaf->node, NULL);
         if (d->leaf->nodetype == LYS_LEAF) {
             ret = lys_compile_unres_leaf_dlft(&cctx, d->leaf, d->dflt, unres);
         } else {
             ret = lys_compile_unres_llist_dflts(&cctx, d->llist, d->dflt, d->dflts, unres);
         }
-        LOG_LOCBACK(1, 0, 0, 0);
+        LOG_LOCBACK(1, 0);
         LY_CHECK_GOTO(ret, cleanup);
 
         lysc_unres_dflt_free(ctx, d);
@@ -1364,13 +1435,9 @@ resolve_all:
     /* finally, remove all disabled nodes */
     for (i = 0; i < ds_unres->disabled.count; ++i) {
         node = ds_unres->disabled.snodes[i];
-        if (node->flags & LYS_KEY) {
-            LOG_LOCSET(node, NULL, NULL, NULL);
-            LOGVAL(ctx, LYVE_REFERENCE, "Key \"%s\" is disabled.", node->name);
-            LOG_LOCBACK(1, 0, 0, 0);
-            ret = LY_EVALID;
-            goto cleanup;
-        }
+        ret = lys_compile_unres_check_disabled(node);
+        LY_CHECK_GOTO(ret, cleanup);
+
         LYSC_CTX_INIT_PMOD(cctx, node->module->parsed, NULL);
 
         lysc_node_free(&cctx.free_ctx, node, 1);
@@ -1390,10 +1457,10 @@ resolve_all:
 
             assert(ret != LY_ERECOMPILE);
             if (ret) {
-                LOG_LOCSET(l->node, NULL, NULL, NULL);
+                LOG_LOCSET(l->node, NULL);
                 LOGVAL(ctx, LYVE_REFERENCE, "Target of leafref \"%s\" cannot be referenced because it is disabled.",
                         l->node->name);
-                LOG_LOCBACK(1, 0, 0, 0);
+                LOG_LOCBACK(1, 0);
                 ret = LY_EVALID;
                 goto cleanup;
             }
@@ -1548,9 +1615,9 @@ lys_compile_unres_mod(struct lysc_ctx *ctx)
             lysc_update_path(ctx, NULL, aug->ext->name);
         }
         lysc_update_path(ctx, NULL, "{augment}");
-        lysc_update_path(ctx, NULL, aug->nodeid->expr);
+        lysc_update_path(ctx, NULL, aug->nodeid->str);
         LOGVAL(ctx->ctx, LYVE_REFERENCE, "Augment%s target node \"%s\" from module \"%s\" was not found.",
-                aug->ext ? " extension" : "", aug->nodeid->expr, LYSP_MODULE_NAME(aug->aug_pmod));
+                aug->ext ? " extension" : "", aug->nodeid->str, LYSP_MODULE_NAME(aug->aug_pmod));
         ctx->cur_mod = orig_mod;
         lysc_update_path(ctx, NULL, NULL);
         lysc_update_path(ctx, NULL, NULL);
@@ -1567,9 +1634,9 @@ lys_compile_unres_mod(struct lysc_ctx *ctx)
     for (i = 0; i < ctx->devs.count; ++i) {
         dev = ctx->devs.objs[i];
         lysc_update_path(ctx, NULL, "{deviation}");
-        lysc_update_path(ctx, NULL, dev->nodeid->expr);
+        lysc_update_path(ctx, NULL, dev->nodeid->str);
         LOGVAL(ctx->ctx, LYVE_REFERENCE, "Deviation(s) target node \"%s\" from module \"%s\" was not found.",
-                dev->nodeid->expr, LYSP_MODULE_NAME(dev->dev_pmods[0]));
+                dev->nodeid->str, LYSP_MODULE_NAME(dev->dev_pmods[0]));
         lysc_update_path(ctx, NULL, NULL);
         lysc_update_path(ctx, NULL, NULL);
     }
@@ -1728,13 +1795,13 @@ lys_compile(struct lys_module *mod, struct lys_depset_unres *unres)
     }
     ctx.pmod = sp;
 
-    LOG_LOCBACK(0, 0, 1, 0);
+    ly_log_location_revert(0, 0, 1, 0);
 
     /* finish compilation for all unresolved module items in the context */
     LY_CHECK_GOTO(ret = lys_compile_unres_mod(&ctx), cleanup);
 
 cleanup:
-    LOG_LOCBACK(0, 0, 1, 0);
+    ly_log_location_revert(0, 0, 1, 0);
     lys_compile_unres_mod_erase(&ctx, ret);
     if (ret) {
         lysc_module_free(&ctx.free_ctx, mod_c);
@@ -1785,7 +1852,7 @@ lys_compile_identities(struct lys_module *mod)
 
 cleanup:
     /* always needed when using lysc_update_path() */
-    LOG_LOCBACK(0, 0, 1, 0);
+    ly_log_location_revert(0, 0, 1, 0);
     return rc;
 }
 
