@@ -23,7 +23,6 @@
 #include <string.h>
 #include <time.h>
 
-#include "common.h"
 #include "compat.h"
 #include "context.h"
 #include "dict.h"
@@ -31,6 +30,7 @@
 #include "in.h"
 #include "in_internal.h"
 #include "log.h"
+#include "ly_common.h"
 #include "parser_schema.h"
 #include "path.h"
 #include "schema_compile.h"
@@ -1001,7 +1001,7 @@ LY_ERR
 lysp_check_stringchar(struct lysp_ctx *ctx, uint32_t c)
 {
     if (!is_yangutf8char(c)) {
-        LOGVAL_PARSER(ctx, LY_VCODE_INCHAR, c);
+        LOGVAL_PARSER(ctx, LY_VCODE_INCHAR, (char)c);
         return LY_EVALID;
     }
     return LY_SUCCESS;
@@ -1014,11 +1014,12 @@ lysp_check_identifierchar(struct lysp_ctx *ctx, uint32_t c, ly_bool first, uint8
         if (!is_yangidentstartchar(c)) {
             if ((c < UCHAR_MAX) && isprint(c)) {
                 if (ctx) {
-                    LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier first character '%c' (0x%04x).", (char)c, c);
+                    LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier first character '%c' (0x%04" PRIx32 ").",
+                            (char)c, c);
                 }
             } else {
                 if (ctx) {
-                    LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier first character 0x%04x.", c);
+                    LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier first character 0x%04" PRIx32 ".", c);
                 }
             }
             return LY_EVALID;
@@ -1034,7 +1035,7 @@ lysp_check_identifierchar(struct lysp_ctx *ctx, uint32_t c, ly_bool first, uint8
         (*prefix) = 1;
     } else if (!is_yangidentchar(c)) {
         if (ctx) {
-            LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier character '%c' (0x%04x).", (char)c, c);
+            LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG, "Invalid identifier character '%c' (0x%04" PRIx32 ").", (char)c, c);
         }
         return LY_EVALID;
     }
@@ -1210,7 +1211,8 @@ search_clb:
                     check_data.name = inc->name;
                     check_data.revision = inc->rev[0] ? inc->rev : NULL;
                     check_data.submoduleof = PARSER_CUR_PMOD(pctx)->mod->name;
-                    lys_parse_submodule(ctx, in, format, pctx, lysp_load_module_check, &check_data, new_mods, &submod);
+                    lys_parse_submodule(ctx, in, format, pctx->main_ctx, lysp_load_module_check, &check_data, new_mods,
+                            &submod);
 
                     /* update inc pointer - parsing another (YANG 1.0) submodule can cause injecting
                      * submodule's include into main module, where it is missing */
@@ -2078,10 +2080,6 @@ lysp_ext_find_definition(const struct ly_ctx *ctx, const struct lysp_ext_instanc
     if (!mod) {
         LOGVAL(ctx, LYVE_REFERENCE, "Invalid prefix \"%.*s\" used for extension instance identifier.", (int)pref_len, prefix);
         return LY_EVALID;
-    } else if (!mod->parsed->extensions) {
-        LOGVAL(ctx, LYVE_REFERENCE, "Extension instance \"%s\" refers \"%s\" module that does not contain extension definitions.",
-                ext->name, mod->name);
-        return LY_EVALID;
     }
 
     if (!ext_def) {
